/*
 * Copyright (c)1998 by Angry Red Planet.
 *
 * This code is distributed under a modified form of the
 * Artistic License.  A copy of this license should have
 * been included with it; if this wasn't the case, the
 * entire package can be obtained at
 * <URL:http://www.angryredplanet.com/>.
 *
 * ----------------------------------------------------------------------
 *
 * ArpConfigurableI.h
 *
 * An abstract interface to an object that is "configurable": it supports
 * setting and getting its parameter state through BMessages, and can
 * create a view to edit these parameters.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * None.
 *
 * ----------------------------------------------------------------------
 *
 * To Do
 * ~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 11/2/1998:
 * 	• Created this file.
 *
 */

#pragma once

#ifndef ARPKERNEL_ARPCONFIGURABLEI_H
#define ARPKERNEL_ARPCONFIGURABLEI_H

#ifndef ARPCOLLECTIONS_ARPVECTORI_H
#include <ArpCollections/ArpVectorI.h>
#endif

enum {
	/**	This is a standard message that configurable objects
	 ** can implement to update their settings.
	 **
	 **	"settings" (B_MESSAGE_TYPE): A BMessage containing
	 ** the new settings values, exactly as passed to
	 ** GetConfiguration().
	 **/
	ARP_PUT_CONFIGURATION_MSG = 'aPCM'
};

class ArpConfigurableI {
public:
	virtual status_t GetConfiguration(BMessage* values) = 0;
	virtual status_t PutConfiguration(BMessage* values) = 0;
	virtual status_t Configure(ArpVectorI<BView*>& views) = 0;
};

#endif
