/*
 * Copyright (c)1998 by Angry Red Planet.
 *
 * This code is distributed under a modified form of the
 * Artistic License.  A copy of this license should have
 * been included with it; if this wasn't the case, the
 * entire package can be obtained at
 * <URL:http://www.angryredplanet.com/>.
 *
 * ----------------------------------------------------------------------
 *
 * ArpDebug.h
 *
 * Helper macros for program debugging...  based on
 * the macros commonly used under AmigaOS.  Use '#define ArpDEBUG 1'
 * to turn on the macros.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * To Do
 * ~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 0.1: Created this file.
 *
 */

#ifndef ARPKERNEL_ARPDEBUG_H
#define ARPKERNEL_ARPDEBUG_H

#if ArpDEBUG
#ifndef DEBUG
#define DEBUG 1
#endif
#endif

#include <assert.h>

#include <be/support/Debug.h>

#define RPTERR(ARGS)	_debugPrintf ARGS

#if ArpDEBUG

#define ArpASSERT(x) assert(x)
#define ArpVALIDATE(x, ret) { assert(x); if( !(x) ) { ret; } }

#ifndef DB_DELAY
#define DB_WAIT(x)
#else
#define DB_WAIT(x) snooze(x)
#endif

#define D(x) x
#define DB(level,x) { if(level) { x; } cdb.flush(); DB_WAIT(DB_DELAY) }
#define DBALL 0xffff
#define cdb cerr
#define stddb stdout
#define bug fprintf

#else

#define ArpASSERT(x)
#define ArpVALIDATE(x, ret) if( !(x) ) { ret; }

#define DB_WAIT(x)
#define D(x)
#define DB(level,x)
#define DBALL 0
#define bug

#endif

#if defined(DEBUG) || defined(USE_STREAMS)
#include <iostream.h>

#ifndef ARPKERNEL_ARPSTRING_H
#include <ArpKernel/ArpString.h>
#endif

#ifndef ARPKERNEL_ARPCOLOR_H
#include <ArpKernel/ArpColor.h>
#endif

// We only define these stream operators when debugging is turned
// on, since using streams bloats code so much...

inline ostream& operator << (ostream& os, const ArpString & str)
{ os << ((const char*)str ? (const char*)str : "<null>"); return os; }

class BPoint;
class BRect;
class BFont;
class BMessenger;
class BMessage;
class BPath;
struct entry_ref;
struct rgb_color;
struct pattern;

ostream& operator << (ostream& os, const BPoint & bp);
ostream& operator << (ostream& os, const BRect & br);
ostream& operator << (ostream& os, const BFont & fn);
ostream& operator << (ostream& os, const BMessenger & o);
ostream& operator << (ostream& os, const BPath & o);
ostream& operator << (ostream& os, const entry_ref & o);
ostream& operator << (ostream& os, const rgb_color & o);
ostream& operator << (ostream& os, const pattern & o);
ostream& operator << (ostream& os, const BMessage & msg);

ostream& ArpToStream(ostream& os, const BMessage& msg, const char* prefix="");

#endif
	
#endif
