/*
 * Copyright (c)1998 by Angry Red Planet.
 *
 * This code is distributed under a modified form of the
 * Artistic License.  A copy of this license should have
 * been included with it; if this wasn't the case, the
 * entire package can be obtained at
 * <URL:http://www.angryredplanet.com/>.
 *
 * ----------------------------------------------------------------------
 *
 * ArpNotificationRegistry.h
 *
 * This file contains a class that is responsible for storing a list of
 * BHandlers with an interest in some piece of data, and sending notices
 * to them.
 * 
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	• In both NotifyDependents() functions, there is now a flag to tell
 * the notify method to deliver asynchronously -- return immediately --
 * or not.  This flag is a PLACE HOLDER.  There is no current capability
 * to actually deliver the messages asynchronously, but it's there for
 * clients who depend upon this behaviour being one way or the other.
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 11.23.98		hackborn
 * Added fake ability to notify dependents asynchronously.  See known bugs.
 */
 
#ifndef ARPKERNEL_ARPNOTIFICATIONREGISTRY_H
#define ARPKERNEL_ARPNOTIFICATIONREGISTRY_H


#include <List.h>
#include <Message.h>
#include <Handler.h>
#ifndef _SUPPORT_DEFS_H
#include <be/support/SupportDefs.h>
#endif


// Values of 0 - 99 are reserved for standard defines.
#define DEP_MISC		(0)


class ArpNotificationRegistry {
	public:
		ArpNotificationRegistry();
		virtual ~ArpNotificationRegistry();

		int32 AddDependent(BHandler *v);
		int32 AddDependent(int32 type, BHandler *v);
		int32 RemoveDependent(BHandler *v);
		int32 RemoveDependent(int32 type, BHandler *v);
		void NotifyDependents(int32 type,
							  BMessage *m,
							  bool deliverAsynchronously = true);

		bool RegisterType(int32 type);
			
	private:
		BList	entries;		// stores ArpDependentEntries
};


// This class is internal to the ArpNotificationRegistry implementation
class ArpNotificationType {
	public:
		ArpNotificationType(int32);
		virtual ~ArpNotificationType();

		void SetType(int32);
		int32 Type();

		bool AddDependent(BHandler*);
		bool RemoveDependent(BHandler*);
		void NotifyDependents(BMessage *msg,
							  bool deliverAsynchronously = true);

	private:
		BList	dependents;		// stores BHandlers*
		int32	type;
};


#endif /* ARP_NOTIFICATION_REGISTRY_H */
