/*
	
	ArpAboutView.cpp
	
	Copyright (c)1998 by Angry Red Planet.

	This code is distributed under a modified form of the
	Artistic License.  A copy of this license should have
	been included with it; if this wasn't the case, the
	entire package can be obtained at
	<URL:http://www.angryredplanet.com/>.
*/

//#define ArpDEBUG 1

#include "ArpKernel/ArpAboutView.h"

#ifndef _WINDOW_H
#include <be/interface/Window.h>
#endif

#ifndef _SCREEN_H
#include <be/interface/Screen.h>
#endif

#ifndef _BUTTON_H
#include <be/interface/Button.h>
#endif

#ifndef _TRANSLATION_UTILS_H
#include <be/translation/TranslationUtils.h>
#endif

#ifndef _APPLICATION_H
#include <be/app/Application.h>
#endif

#ifndef _APPFILEINFO_H
#include <be/storage/AppFileInfo.h>
#endif

#ifndef _FILE_H
#include <be/storage/File.h>
#endif

#ifndef ARPKERNEL_ARPDEBUG_H
#include <ArpKernel/ArpDebug.h>
#endif

#include <stdio.h>

ArpAboutView::ArpAboutView(BRect rect, const char *name, const char *text,
						const char* appname, const char* verstr, const char* build)
	   	   : BView(rect, name, B_FOLLOW_ALL,
	   	   			B_WILL_DRAW|B_FRAME_EVENTS|B_FULL_UPDATE_ON_RESIZE),
	   	     N(0), mAppPic(0), mArpPic(0), mBitmap(0), mDrawView(0), mStep(0),
	   	     mAnimBaseTime(system_time()), mAnimThread(-1)
{
	ArpStrTok tok(ArpLIT(text), ArpLIT("\r\n"));
	ArpString curStr;
	while( ! (curStr=tok.Next()).IsNull() ) {
		mText.push_back(curStr);
		printf("Got line: %s\n", (const char*)(mText[mText.size()-1]));
	}
	
	N = mText.size();
	
	mAppName = appname;
	mVersion = verstr;
	mBuild = build;
	
	{
		app_info ai;
		BFile file;
		BAppFileInfo afi;
		be_app->GetAppInfo(&ai);
		file.SetTo(&ai.ref, B_READ_ONLY);
		afi.SetTo(&file);
		mAppPic = new BBitmap(BRect(0,0,31,31), B_CMAP8);
		afi.GetIcon(mAppPic, B_LARGE_ICON);
	}
	
	mArpPic = BTranslationUtils::GetBitmap('jpeg', 1);
	
	mDrawMode = 0xffffffff;
	mBaseFont = be_bold_font;
	mBaseFont.SetSize(14);
	mOutliers = 20;
	mFlatPart = .1;
	mStep = -mOutliers;
	
	ComputeDimens();
	
	mDoneBut = new BButton(BRect(0,0,100,20), "done",
						  "Go Away",
						  new BMessage(B_QUIT_REQUESTED),
						  B_FOLLOW_BOTTOM | B_FOLLOW_LEFT);
	float doneW=0, doneH=0;
	mDoneBut->GetPreferredSize(&doneW, &doneH);
	mDoneBut->ResizeToPreferred();
	mDoneBut->MoveTo(5, Bounds().bottom-doneH-5);
	AddChild(mDoneBut);
	
	mAnimThread = spawn_thread(animEntry, "ARP™ Animator",
								B_NORMAL_PRIORITY, this);
}

ArpAboutView::~ArpAboutView()
{
	if( mAnimThread >= 0 ) {
		status_t ret;
		kill_thread(mAnimThread);
		wait_for_thread(mAnimThread,&ret);
		mAnimThread = -1;
	}
	delete mAppPic;
	delete mArpPic;
	delete mBitmap;
}

void ArpAboutView::AttachedToWindow()
{
	inherited::AttachedToWindow();
	BView* par = Parent();
	if( par ) SetViewColor(par->ViewColor());
	else SetViewColor(0, 0, 0);
	Window()->SetDefaultButton(mDoneBut);
	mDoneBut->SetTarget(Window());
	if( mAnimThread >= 0 ) {
		resume_thread(mAnimThread);
	}
}

void ArpAboutView::DetachedFromWindow()
{
	inherited::DetachedFromWindow();
	if( mAnimThread >= 0 ) {
		suspend_thread(mAnimThread);
	}
}

void ArpAboutView::GetPreferredSize(float* width, float* height)
{
	if( width ) *width = 500;
	if( height ) *height = 300;
}

float ArpAboutView::FindMaxWidth(const BFont& font)
{
	float val = 0;
	size_t i;
	for( i=0; i<mText.size(); i++ ) {
		if( !mText[i].IsEmpty() ) {
			float w = font.StringWidth((const char*)(mText[i]));
			if( w > val ) val = w;
		}
	}
	return val;
}

void ArpAboutView::ComputeDimens()
{
	mBaseSize = mBaseFont.Size();
	mRotPart = 1.0-mFlatPart;
	mFlatPos = mFlatPart*mOutliers;
	mFlatPixel = mFlatPos*mBaseSize;
	mTextCenter.x = mArpPic ? (mArpPic->Bounds().right+10) : 30;
	mTextCenter.y = Bounds().top + (Bounds().bottom-Bounds().top)/2;
	mYRot = (mArpPic
				? (mArpPic->Bounds().bottom/2)
				: (Bounds().bottom/2)
			) - (mFlatPixel/2);
	if( mYRot < mFlatPixel*4 ) mYRot = mFlatPixel*4;
	
	DB(DBALL, cdb << "ComputeDimens():" << endl
					<< "  mBaseSize=" << mBaseSize
					<< ", mFlatPart=" << mFlatPart
					<< ", mRotPart=" << mRotPart << endl
					<< "  mFlatPos=" << mFlatPos
					<< ", mFlatPixel=" << mFlatPixel << endl
					<< "  mTextCenter=" << mTextCenter
					<< ", mYRot=" << mYRot << endl);
}

void ArpAboutView::FrameResized(float new_width, float new_height)
{
	ComputeDimens();
}

#if 0
void ArpAboutView::DrawLine(const char* text, float pos, float center, BFont& font)
{
	if( !mDrawView ) return;
	
	const float ABSPOS = fabs(pos);
	const float ABSREL = ABSPOS/OUTLIERS;
	const float SIGN = pos < 0 ? -1 : 1;
	const float TOTDIST = (OUTLIERS-OUTLIERS/2+OUTLIERS/4) * FONTSIZE;
	
	font.SetShear( 90 - 80.0*sin(pos/OUTLIERS*M_PI_2) );
	//font.SetRotation( -90.0*ABSREL*SIGN );
	font.SetSize( FONTSIZE - ( (FONTSIZE-4)*ABSREL ) );
	mDrawView->SetFont(&font,B_FONT_SIZE|B_FONT_SHEAR);
	mDrawView->SetHighColor(255,255-int(255*ABSREL),0);
	const float STRWID = font.StringWidth(text);
	const float HALFX = mDrawView->Bounds().right/2;
	BPoint point;
	point.x = HALFX - (STRWID/2) + sin(pos/OUTLIERS*M_PI_2)*HALFX;
	//const float yoff = -TOTDIST + cos(ABSPOS/OUTLIERS)*TOTDIST;
	const float yoff = (ABSPOS-ABSPOS/2+ABSPOS/4)*(FONTSIZE) * SIGN;
	point.y = center + yoff;
	mDrawView->DrawString(text, point);
}
#endif

#if 0
void ArpAboutView::DrawLine(const char* text, float pos, float center, BFont& font)
{
	if( !mDrawView ) return;
	
	if( pos < -OUTLIERS || pos > OUTLIERS ) return;
	//if( pos < -OUTLIERS ) pos = -OUTLIERS;
	//if( pos > OUTLIERS ) pos = OUTLIERS;
	
	const float ABSPOS = fabs(pos);
	const float ABSREL = ABSPOS/OUTLIERS;
	const float SIGN = pos < 0 ? -1 : 1;
	//const float TOTDIST = (OUTLIERS-OUTLIERS/2+OUTLIERS/4) * FONTSIZE;
	const float XOFF = mArpPic ? mArpPic->Bounds().right : 30;
	#if 1
	const float YOFF = (mArpPic
							? (mArpPic->Bounds().bottom/2)
							: (Bounds().bottom/2)
						) - (.2*OUTLIERS);
	#else
	const float YOFF = Bounds().bottom - (.2*OUTLIERS);
	#endif
	
	//font.SetRotation( -90.0*sin(pos/OUTLIERS*M_PI_2) );
	BPoint point;
	const float OFFREL = ABSREL > .2 ? ((ABSREL-.2)/.8) : 0;
	//font.SetRotation( -90.0*(1-sqrt(1-OFFREL))*SIGN );
	//font.SetSize( floor( (FONTSIZE - ( (FONTSIZE-4)*OFFREL) ) * 400 ) / 400 );
	#if 1
	if( ABSREL > .2 ) {
		//font.SetRotation( -90.0*(1-sqrt(1-OFFREL))*SIGN );
		font.SetRotation( -90*(ABSREL-.2)/.8*SIGN );
		font.SetSize( FONTSIZE - ( (FONTSIZE-2)*OFFREL ) );
		const float SIGNOFF = .2*SIGN*OUTLIERS;
		point.x = -10 + cos((pos-SIGNOFF)/(OUTLIERS-.2*OUTLIERS)*M_PI_2)*(XOFF+20);
		const float OFFPOS = .2*OUTLIERS;
		//const float yoff = ( OFFPOS*FONTSIZE + ((ABSPOS-OFFPOS)-(ABSPOS-OFFPOS)/2
		//								+(ABSPOS-OFFPOS)/4)*(FONTSIZE) ) * SIGN;
		const float yoff = (OFFPOS*FONTSIZE*SIGN)
							+ sin((pos-SIGNOFF)/(OUTLIERS-.2*OUTLIERS)*M_PI_2)*YOFF;
		point.y = center + yoff;
	} else {
		font.SetRotation( 0 );
		font.SetSize( FONTSIZE );
		//const float SIGNOFF = .2*SIGN;
		//point.x = -10 + cos((.2*SIGN-SIGNOFF)/(OUTLIERS-SIGNOFF)*M_PI_2)*(XOFF+20);
		point.x = -10 + (XOFF+20);
		const float yoff = ABSPOS*FONTSIZE*SIGN;
		point.y = center + yoff;
	}
	#endif
	point.x = -10 + cos(pos/OUTLIERS*M_PI_2)*(XOFF+20);
	//font.SetRotation(0);
	//point.x = XOFF;
	//font.SetRotation( -90.0*ABSREL*SIGN );
	mDrawView->SetFont(&font,B_FONT_SIZE|B_FONT_ROTATION);
	mDrawView->SetHighColor(255,255-int(255*ABSREL),0);
	//point.x = -10 + cos(pos/OUTLIERS*M_PI_2)*(XOFF+20);
	//const float yoff = -TOTDIST + cos(ABSPOS/OUTLIERS)*TOTDIST;
	//const float yoff = (ABSPOS-sin(ABSREL*M_PI_4)*ABSPOS)*(FONTSIZE) * SIGN;
	//const float yoff = (ABSPOS-ABSPOS/3+ABSPOS/4)*(FONTSIZE) * SIGN;
	//point.y = center + yoff;
	if( mDrawMode&2 ) {
		float w = font.StringWidth(text);
		BPoint endp(point);
		endp.x += w*cos(font.Rotation()/180.0*M_PI);
		endp.y -= w*sin(font.Rotation()/180.0*M_PI);
		mDrawView->StrokeLine(point, endp);
	}
	if( mDrawMode&1 ) {
		mDrawView->DrawString(text, point);
	}
	if( 0 && text == (const char*)(mText[0]) ) {
		printf("Pos %f: Rotation=%f, Size=%f, pnt=(%f, %f)\n",
				pos, font.Rotation(), font.Size(), point.x, point.y);
	}
}
#endif

#if 1
void ArpAboutView::DrawLine(const char* text, float pos, float center, BFont& font)
{
	if( !mDrawView ) return;
	
	if( pos < -mOutliers || pos > mOutliers ) return;
	
	const float ABSREL = fabs(pos)/mOutliers;
	const float SIGN = pos < 0 ? -1 : 1;
	const float ROTREL = ABSREL > mFlatPart
					? ((ABSREL-mFlatPart)/mRotPart) : 0;

	BPoint point(mTextCenter);
	//font.SetRotation( -90*ROTREL*SIGN );
	//font.SetShear( 90 + 45*ROTREL*SIGN );
	font.SetRotation(0);
	font.SetSize( mBaseSize - ( (mBaseSize-1)*ROTREL ) );
	//point.x = -10 + cos(pos/mOutliers*M_PI_2)*(point.x+10);
	point.x = Bounds().right - (-10 + cos(pos/mOutliers*M_PI_2)*(point.x+10));
	if( ABSREL > mFlatPart ) {
		point.y += mFlatPixel*SIGN
					+ sin((pos-(mFlatPos*SIGN))/(mOutliers-mFlatPos)*M_PI_2)*mYRot;
	} else {
		point.y += pos*mBaseSize;
	}
	
	mDrawView->SetFont(&font,B_FONT_SIZE|B_FONT_ROTATION|B_FONT_SHEAR);
	if( ABSREL < .5 ) {
		mDrawView->SetHighColor(255,255-int(255*ABSREL/.5),0);
	} else {
		mDrawView->SetHighColor(255-int(255*(ABSREL-.5)),0,0);
	}
	
	float w = font.StringWidth(text);
	mDrawMode = 1;
	if( mDrawMode&2 ) {
		BPoint endp(point);
		endp.x += w*cos(font.Rotation()/180.0*M_PI);
		endp.y -= w*sin(font.Rotation()/180.0*M_PI);
		mDrawView->StrokeLine(point, endp);
	}
	if( mDrawMode&1 ) {
		point.x -= w;
		mDrawView->DrawString(text, point);
	}
#if 0 && ArpDEBUG
	if( text == (const char*)(mText[0]) ) {
		DB(DBALL, cdb << "Pos " << pos << ": Rotation=" << font.Rotation()
						<< ", Size=" << font.Size()
						<< ", Pnt=" << point << endl);
	}
#endif
}
#endif

void ArpAboutView::Draw(BRect)
{
	// Set up the backing store bitmap.
	{
		BScreen screen(Window());
		if( !mBitmap || mBitmap->Bounds() != Bounds()
			|| mBitmap->ColorSpace() != screen.ColorSpace() ) {
			Sync();
			delete mBitmap;
			mBitmap = 0;
			mDrawView = 0;
			mBitmap = new BBitmap(Bounds(), screen.ColorSpace(), true);
			mDrawView = new BView(Bounds(), "BackingStore",
								  B_FOLLOW_ALL, B_WILL_DRAW);
			mBitmap->AddChild(mDrawView);
		}
	}
	
	if( !mBitmap || !mDrawView ) return;
	if( !mBitmap->Lock() ) return;
	
	int i;
	const int CURLINE = int(mStep);
	
	BRect bounds = mDrawView->Bounds();
	
	mDrawView->SetLowColor(0,0,0);
	mDrawView->SetHighColor(0,0,0);
	
#if 0
	printf("Step: %f, Bounds: (%f,%f)-(%f,%f)\n", mStep,
			bounds.left, bounds.top, bounds.right, bounds.bottom);
#endif

	mDrawView->FillRect(bounds);
	BPoint pos(5,5);
	float pich = 0;
	if( mAppPic ) {
		mDrawView->SetDrawingMode(B_OP_OVER);
		mDrawView->DrawBitmapAsync(mAppPic, pos);
		mDrawView->SetDrawingMode(B_OP_COPY);
		pos.x += mAppPic->Bounds().right + 5;
		pich = mAppPic->Bounds().bottom;
	}
	BFont curFont(&mBaseFont);
	curFont.SetSpacing(B_BITMAP_SPACING);
	curFont.SetSize(32);
	curFont.SetFace(B_BOLD_FACE);
	font_height height;
	curFont.GetHeight(&height);
	//pos.y += height.ascent;
	pos.y += height.ascent + (pich-height.ascent-height.descent)/2;
	mDrawView->SetFont(&curFont);
	mDrawView->SetHighColor(255,255,255);
	mDrawView->DrawString(mAppName, pos);
	
	if( mArpPic ) {
		//pos.x = bounds.right-mArpPic->Bounds().right;
		//pos.y = 0;
		pos.x = bounds.right-mArpPic->Bounds().right-5;
		pos.y = (bounds.bottom/2)-(mArpPic->Bounds().bottom/2);
		mDrawView->DrawBitmapAsync(mArpPic, pos);
	}
	
	curFont.SetSpacing(B_CHAR_SPACING);
	mDrawView->SetFont(&curFont);
	
	for( i=-mOutliers; i<=mOutliers; i++ ) {
		const size_t line = CURLINE+i;
		if( line >= 0 && line < N ) {
			DrawLine((const char*)(mText[line]), (int(mStep)-mStep+i),
					 bounds.top+ (bounds.bottom-bounds.top)/2, curFont);
		}
	}
	
	mDrawView->Sync();
	DrawBitmapAsync(mBitmap);
	mBitmap->Unlock();
}

void ArpAboutView::MouseDown(BPoint)
{
	mDrawMode++;
}

void ArpAboutView::StepAnim()
{
	mStep = float(system_time()-mAnimBaseTime)/700000 - mOutliers;
	if( mStep > (N+mOutliers) ) mAnimBaseTime = system_time();
	Draw(Bounds());
}

int32 ArpAboutView::animEntry(void *arg) 
{ 
	ArpAboutView *obj = (ArpAboutView *)arg; 
	return (obj->animFunc()); 
}

int32 ArpAboutView::animFunc()
{
	while( 1 ) {
		bigtime_t nexttime = system_time() + 50000;
		if( LockLooper() ) {
			StepAnim();
			UnlockLooper();
		}
		if( nexttime < system_time() ) nexttime = system_time() + 1000;
		snooze_until(nexttime, B_SYSTEM_TIMEBASE);
	}
}
