/*
	
	ArpViewWrapper.cpp
	
	Copyright (c)1998 by Angry Red Planet.

	This code is distributed under a modified form of the
	Artistic License.  A copy of this license should have
	been included with it; if this wasn't the case, the
	entire package can be obtained at
	<URL:http://www.angryredplanet.com/>.

	An ArpLayoutable that can be used to wrap around another
	BView, without requiring multiple inheritance or subclassing.
*/

//#define ArpDEBUG 1

#ifndef ARPLAYOUT_ARPVIEWWRAPPER_H
#include "ArpLayout/ArpViewWrapper.h"
#endif

#ifndef ARPKERNEL_ARPDEBUG_H
#include "ArpKernel/ArpDebug.h"
#endif

#include <float.h>
#include <algobase.h>

#include <be/support/Autolock.h>

static const char* ViewParam = "ArpViewWrapper::View";

/* ------ ArpViewWrapper constructor and destructor ------
 *
 * The various ways to create and destroy ArpViewWrapper
 * objects.
 */
 
ArpViewWrapper::ArpViewWrapper(BView* wrapview)
	: ArpLayoutable(wrapview ? wrapview->Name() : (const char*)0),
	  view(wrapview)
{
	own_view = true;
	if( !view ) {
		SetError(B_NO_MEMORY);
		return;
	}
}

ArpViewWrapper::ArpViewWrapper(BMessage* data, BView* v)
	: ArpLayoutable(data), view(v)
{
	own_view = true;
	if( data ) {
		BMessage viewarch;
		if( !view &&
			data->FindMessage(ViewParam,&viewarch) == B_NO_ERROR ) {
			view = cast_as(instantiate_object(&viewarch),BView);
		}
		if( !view ) SetError(B_NO_MEMORY);
	}
}

/* ------------ ArpViewWrapper archiving ------------
 *
 * Archiving and retrieving ArpViewWrapper objects.
 */
ArpViewWrapper* ArpViewWrapper::Instantiate(BMessage* archive)
{
	if ( validate_instantiation(archive, "ArpViewWrapper") ) 
		return new ArpViewWrapper(archive); 
	return NULL;
}

status_t ArpViewWrapper::Archive(BMessage* data, bool deep)
{
	status_t status = ArpLayoutable::Archive(data,deep);
	if( status == B_NO_ERROR )
		status = data->AddString("class","ArpViewWrapper");
	if( status == B_NO_ERROR && view && own_view ) {
		BMessage viewarch;
		status = view->Archive(&viewarch,false);
		if( status == B_NO_ERROR )
			status = data->AddMessage(ViewParam,&viewarch);
	}
	return status;
}
 
void ArpViewWrapper::OwnView(void)
{
	own_view = true;
}

BView* ArpViewWrapper::OwnerView(void)
{
	return view;
}

void ArpViewWrapper::ComputeDimens(void)
{
	inherited::ComputeDimens();
	DB(DBALL,printf("**** %s\n",LayoutName()));
	DB(DBALL,printf("orig min_width = %f, min_height = %f\n",
		cur_dimens.min_width, cur_dimens.min_height));
	DB(DBALL,printf("orig pref_width = %f, pref_height = %f\n",
		cur_dimens.pref_width, cur_dimens.pref_height));
	float vw=0;
	float vh=0;
	if( view ) {
		// XXX This is an UGLY, UGLY hack!!  But it seems we
		// have to do this to get most views to return reasonable
		// dimensions to us...
		BRect frm = view->Frame();
		DB(DBALL,cdb << "Got current frame: " << frm << endl);
		view->ResizeTo(1,1);
		DB(DBALL,cdb << "Resized to (1,1)" << endl);
		view->Draw(view->Bounds());
		DB(DBALL,cdb << "Called Draw() using bounds dimensions." << endl);
		//view->Invalidate();
		view->ResizeToPreferred();
		DB(DBALL,cdb << "Resized to preferred." << endl);
		view->GetPreferredSize(&vw,&vh);
		DB(DBALL,cdb << "retrieved preferred size." << endl);
		vw++;
		vh++;
		view->ResizeTo(frm.Width(),frm.Height());
		DB(DBALL,printf("vwidth = %f, vheight = %f\n",vw,vh));
	}
	cur_dimens.min_width = max(cur_dimens.min_width,vw);
	cur_dimens.min_height = max(cur_dimens.min_height,vh);
	cur_dimens.pref_width = max(cur_dimens.pref_width,vw);
	cur_dimens.pref_height = max(cur_dimens.pref_height,vh);
	cur_dimens.max_width = cur_dimens.pref_width;
	cur_dimens.max_height = cur_dimens.pref_height;
	DB(DBALL,printf("min_width = %f, min_height = %f\n",
		cur_dimens.min_width, cur_dimens.min_height));
	DB(DBALL,printf("pref_width = %f, pref_height = %f\n",
		cur_dimens.pref_width, cur_dimens.pref_height));
}
